==> xmapcore_1.2.8.tar.gz: xmapcore/R/db.R <==
.make.hash = function() {
	new.env( hash=TRUE, parent=emptyenv() )
}

# A global holding our internal state (as a hash, as hashes are mutable in R)
.xmap.internals = .make.hash()

# Exported functions
xmap.connect = function( name ) {
	if( is.null( .xmap.internals$initialised ) ) {
		.initialise()
	}
	.xmap.internals$procs$connect( name )
}

xmap.disconnect = function() {
	.xmap.internals$procs$disconnect()
}

# Internal functions
.xmc.connect = function( name ) {
	f = .xmap.internals$conf.dir
	f = file.path( f, "databases.txt" )
	if( !file.exists( f ) ) {
		stop( paste( "Can't find database config file '",
						f,
						"'. See package installation instructions for help setting this up.",
						sep="" ) )
	}
	a = read.table( f, sep="\t", strip.white=TRUE, header=TRUE )
	if( !all( colnames( a ) == c( "name", "host", "species", "version", "port", "username", "password" ) ) ) {
		stop( paste( "Unexpected column names in", 
						f,
						" - should be 'name', 'host', 'species', 'version', 'port', 'username' and 'password'" ) )
	}
	if( missing( name ) ) {
		r = 1
		if( dim( a )[1] > 1 ) {
			choices = paste( as.character( a[,1] ), " -- ", as.character( a[,3] ), " v", as.character( a[,4] ), " (", as.character( a[,2] ), ")", sep="" )
			r = menu( choices, title="Select a database to connect to:" )
		}
		if( r == 0 ) {
			return( invisible() )
		}
		else {
			name = a[,1][r]
		}
	}
	dbs = a[ ( as.character( a$name ) == name ), , drop=FALSE ]
	if( dim( dbs )[1] == 0 ) {
		stop( paste( "Invalid db name:", name ) )
	}
	un = .get.user.string( as.character( dbs[,"username"] ), "username" )
	if( is.na( dbs[,"password"] ) ) {
		pw = ""
	}
	else {
		pw = .get.user.string( as.character( dbs[,"password"] ), "password" )
	}
	xmap.disconnect()
	species = as.character( dbs[,"species"] )
	version = as.character( dbs[,"version"] )
	db.name = paste( "xmapcore_", species, "_", version, sep="" )
	host = as.character( dbs[,"host"] )
	port = as.integer( dbs[,"port"] )
	if( is.na( port ) ) {
		con = dbConnect( "MySQL", user=un, password=pw, dbname=db.name, host=host, client.flag=131072 )
	}
	else {
		con = dbConnect( "MySQL", user=un, password=pw, dbname=db.name, host=host, port=port, client.flag=131072 )
	}
	.xmap.internals$con       = con
	.xmap.internals$species   = species
	.xmap.internals$version   = version
	.xmap.internals$host      = host
	.xmap.internals$port      = port
	.xmap.internals$db.name   = db.name
	.xmap.internals$connected = TRUE

	cat( paste( 'Connected to ', .xmap.internals$db.name, ' (', .xmap.internals$host, ')\n', sep='' ) )

	xmap.array.type( NULL, pick.default=TRUE )

	invisible( list( host=host, species=species, version=version, port=port, username=un, password=pw ) )
}

.xmc.disconnect = function() {
	if( !is.null( .xmap.internals$con ) ) {
		cat( paste( 'Disconnecting from ', .xmap.internals$db.name, ' (', .xmap.internals$host, ')\n', sep='' ) )
		dbDisconnect( .xmap.internals$con )
		tryCatch( rm( 'con',       envir=.xmap.internals ), warning=function(a){invisible()} )
		tryCatch( rm( 'species',   envir=.xmap.internals ), warning=function(a){invisible()} )
		tryCatch( rm( 'array',     envir=.xmap.internals ), warning=function(a){invisible()} )
		tryCatch( rm( 'db.name',   envir=.xmap.internals ), warning=function(a){invisible()} )
	}
	.xmap.internals$connected = FALSE
}

.xmc.db.cleanup = function( conn ) {
	res = dbListResults( conn )
	for( i in res )  {
		if( !dbHasCompleted( i ) ) {
			fetch( i, -1 )
			dbClearResult( i )
		}
	}
	while( dbMoreResults( conn ) ) {
		res = dbNextResult( conn )
		if( !dbHasCompleted( res ) ) {
			fetch( res, -1 )
			dbClearResult( res )
		}
	}
}

.xmc.db.call = function( sql, conn ) {
	.xmap.internals$debugFn( sql )
	on.exit( .xmc.db.cleanup( conn ) )
	res  = dbSendQuery( conn, sql )
	d    = fetch( res, -1 )
	if( length( d ) == 0 ) {
		d = NULL
	}
	d
}

.build.sql2 = function( stmt, hash ) {
	if( !is.null( hash ) ) {
		vars = ls( envir=hash )
		for( var in vars ) {
			val = get( var, envir=hash )
			patt = paste( "\\$\\{", var, "\\}", sep="" )
			stmt = gsub( patt, val, stmt )
		}
	}
	stmt
}

==> xmapcore_1.6.2.tar.gz: xmapcore/R/db.R <==
.make.hash = function() {
	new.env( hash=TRUE, parent=emptyenv() )
}

# A global holding our internal state (as a hash, as hashes are mutable in R)
.xmap.internals = .make.hash()

# Exported functions
xmap.connect = function( name ) {
	if( is.null( .xmap.internals$initialised ) ) {
		.initialise()
	}
	.xmap.internals$procs$connect( name )
}

xmap.disconnect = function() {
	.xmap.internals$procs$disconnect()
}

# Internal functions
.xmc.connect = function( name ) {
	f = .xmap.internals$conf.dir
	f = file.path( f, "databases.txt" )
	if( !file.exists( f ) ) {
		stop( paste( "Can't find database config file '",
						f,
						"'. See package installation instructions for help setting this up.",
						sep="" ) )
	}
	a = read.table( f, sep="\t", strip.white=TRUE, header=TRUE )
	if( !all( colnames( a ) == c( "name", "host", "species", "version", "port", "username", "password" ) ) ) {
		stop( paste( "Unexpected column names in", 
						f,
						" - should be 'name', 'host', 'species', 'version', 'port', 'username' and 'password'" ) )
	}
	if( missing( name ) ) {
		r = 1
		if( dim( a )[1] > 1 ) {
			choices = paste( as.character( a[,1] ), " -- ", as.character( a[,3] ), " v", as.character( a[,4] ), " (", as.character( a[,2] ), ")", sep="" )
			r = menu( choices, title="Select a database to connect to:" )
		}
		if( r == 0 ) {
			return( invisible() )
		}
		else {
			name = a[,1][r]
		}
	}
	dbs = a[ ( as.character( a$name ) == name ), , drop=FALSE ]
	if( dim( dbs )[1] == 0 ) {
		stop( paste( "Invalid db name:", name ) )
	}
	un = .get.user.string( as.character( dbs[,"username"] ), "username" )
	if( is.na( dbs[,"password"] ) ) {
		pw = ""
	}
	else {
		pw = .get.user.string( as.character( dbs[,"password"] ), "password" )
	}
	xmap.disconnect()
	species = as.character( dbs[,"species"] )
	version = as.character( dbs[,"version"] )
	db.name = paste( "xmapcore_", species, "_", version, sep="" )
	host = as.character( dbs[,"host"] )
	port = as.integer( dbs[,"port"] )
	if( is.na( port ) ) {
		con = dbConnect( MySQL(), user=un, password=pw, dbname=db.name, host=host, client.flag=131072 )
	}
	else {
		con = dbConnect( MySQL(), user=un, password=pw, dbname=db.name, host=host, port=port, client.flag=131072 )
	}
	.xmap.internals$con       = con
	.xmap.internals$species   = species
	.xmap.internals$version   = version
	.xmap.internals$host      = host
	.xmap.internals$port      = port
	.xmap.internals$db.name   = db.name
	.xmap.internals$connected = TRUE

	cat( paste( 'Connected to ', .xmap.internals$db.name, ' (', .xmap.internals$host, ')\n', sep='' ) )

	xmap.array.type( NULL, pick.default=TRUE )

	invisible( list( host=host, species=species, version=version, port=port, username=un, password=pw ) )
}

.xmc.disconnect = function() {
	if( !is.null( .xmap.internals$con ) ) {
		cat( paste( 'Disconnecting from ', .xmap.internals$db.name, ' (', .xmap.internals$host, ')\n', sep='' ) )
		dbDisconnect( .xmap.internals$con )
		tryCatch( rm( 'con',       envir=.xmap.internals ), warning=function(a){invisible()} )
		tryCatch( rm( 'species',   envir=.xmap.internals ), warning=function(a){invisible()} )
		tryCatch( rm( 'array',     envir=.xmap.internals ), warning=function(a){invisible()} )
		tryCatch( rm( 'db.name',   envir=.xmap.internals ), warning=function(a){invisible()} )
	}
	.xmap.internals$connected = FALSE
}

.xmc.db.cleanup = function( conn ) {
	res = dbListResults( conn )
	for( i in res )  {
		if( !dbHasCompleted( i ) ) {
			fetch( i, -1 )
			dbClearResult( i )
		}
	}
	while( dbMoreResults( conn ) ) {
		res = dbNextResult( conn )
		if( !dbHasCompleted( res ) ) {
			fetch( res, -1 )
			dbClearResult( res )
		}
	}
}

.xmc.db.call = function( sql, conn ) {
	.xmap.internals$debugFn( sql )
	on.exit( .xmc.db.cleanup( conn ) )
	res  = dbSendQuery( conn, sql )
	d    = fetch( res, -1 )
	if( length( d ) == 0 ) {
		d = NULL
	}
	d
}

.build.sql2 = function( stmt, hash ) {
	if( !is.null( hash ) ) {
		vars = ls( envir=hash )
		for( var in vars ) {
			val = get( var, envir=hash )
			patt = paste( "\\$\\{", var, "\\}", sep="" )
			stmt = gsub( patt, val, stmt )
		}
	}
	stmt
}
